use poem::http::StatusCode;
use serde_json::json;
use shit_post_db::{entities::*,*};
use telegram_bot::TelegramBot;

use poem::error::InternalServerError;
use poem::web::{Data, Form, Html, Path, IntoResponse, Json};
use poem::{handler, Result, Response};
use serde::{Serialize, Deserialize};
use redis;

use telegram_bot::telegram_types::TelegramUpdate;
use shit_post_db::DatabaseConnection;

#[derive(Debug, Clone)]
pub struct AppState {
    pub conn: DatabaseConnection,
    pub templates: tera::Tera,
    pub redis_client: redis::Client,
    pub bot_token: String
}
#[derive(Serialize, Debug)]
pub struct ShitpostResponce {
    title: String,
    url: String,
}

#[derive(Serialize, Debug)]
pub struct ShitpostTypesResponce {
    id: i32,
    title: String
}

#[derive(Deserialize, Debug)]
pub struct ShitpostPostRequest {
    title: String,
    type_id: i32,
    url: String,
    seconds: i32,
}

#[handler]
pub async fn get_video(state: Data<&AppState>, Path(id): Path<i32>) -> impl IntoResponse {
    match shit_post_db::get_random_video_entry_of_type(&state.conn, id).await {
        Ok(video)=> {
            Response::builder()
                .status(StatusCode::OK)
                .content_type("application/json")
                .body(
                    json!(ShitpostResponce { title: video.title, url: video.url }).to_string()
                )

        },
        Err(_) => {
            print!("ERROR");
            Response::builder()
            .status(StatusCode::NOT_FOUND)
            .body(())
        }
    }
}

#[handler]
pub async fn post_shitpost(
    state: Data<&AppState>, Form(req): Form<ShitpostPostRequest>
) -> impl IntoResponse {
    match create_video(&state.conn, &req.title, req.type_id, &req.url, req.seconds,).await {
        Ok(_) => Response::builder().status(StatusCode::OK).body(()),
        Err(_) => {
            print!("ERROR");
            Response::builder()
            .status(StatusCode::BAD_REQUEST)
            .body(())
        }
    }

}

#[handler]
pub async fn index(state: Data<&AppState>) -> Result<impl IntoResponse> {
    let ctx = tera::Context::new();
    let body = state
        .templates
        .render("index.html.tera", &ctx)
        .map_err(InternalServerError)?;
    Ok(Html(body))
}

#[handler]
pub async fn shitpost_view(state: Data<&AppState>) -> Result<impl IntoResponse> {
    let mut ctx = tera::Context::new();



    ctx.insert("table_head", &vec!["id", "title", ""]);
    let body = state
        .templates
        .render("add_shitpost.html.tera", &ctx)
        .map_err(InternalServerError)?;
    Ok(Html(body))
}

#[handler]
pub async fn add_shitpost_view(state: Data<&AppState>) -> Result<impl IntoResponse> {
    let mut ctx = tera::Context::new();
    let types: JsonValue = match shit_post_db::get_shitpost_types(&state.conn).await {
        Ok(types)=> {
            json!(
                types.into_iter().map(
                    |item: entities::types::Model| {
                        ShitpostTypesResponce {
                            id: item.id,
                            title: item.title
                        }
                    }
                ).collect::<Vec<ShitpostTypesResponce>>()
            )
        },
        Err(_) => json!([" "])
    };

    ctx.insert("types", &types);
    let body = state
        .templates
        .render("add_shitpost.html.tera", &ctx)
        .map_err(InternalServerError)?;
    Ok(Html(body))
}

#[handler]
pub async fn get_shitpost_types(state: Data<&AppState>) -> impl IntoResponse {
    match shit_post_db::get_shitpost_types(&state.conn).await {
        Ok(types)=> {
            Response::builder()
                .status(StatusCode::OK)
                .content_type("application/json")
                .body(
                    json!(
                        types.into_iter().map(
                            |item: entities::types::Model| {
                                ShitpostTypesResponce {
                                    id: item.id,
                                    title: item.title
                                }
                            }
                        ).collect::<Vec<ShitpostTypesResponce>>()
                    ).to_string()
                )
        },
        Err(_) => {
            print!("ERROR");
            Response::builder()
            .status(StatusCode::NOT_FOUND)
            .body(())
        }
    }
}

fn _prepare_video(video: videos::Model) -> String {
    format!(
        "-----
        Id: {:}
        Название: {:}
        Ссылка: {:}
        Продолжительность: {:} секунд
        Просмотров: {:}
        -----",
        video.id, video.title, video.url,
        video.duration.unwrap_or_default(), video.access_count.unwrap_or_default()
    )
}

#[handler]
pub async fn tg_bot_handler(state: Data<&AppState>, req: Json<TelegramUpdate>) -> impl IntoResponse {


    let tg_bot = TelegramBot::new(state.bot_token.to_owned(), state.redis_client.to_owned());

    if tg_bot.check_if_handled(req.update_id).unwrap() {
        return Response::builder()
            .status(StatusCode::ALREADY_REPORTED)
            .body(());
    }

    let types: Vec<types::Model> = types::Entity
        .select()
        .all(&state.conn).await.unwrap();

    let kb_markup = telegram_bot::telegram_types::TelegramKeyboardMarkup {
        keyboard: vec![
            types.clone().into_iter().map(
                |model|  format!("{:}",model.title)
            ).collect()
        ],
        resize_keyboard: true,
    };

    // let type_model = match types::Entity
    //     .select().filter(
    //         types::Column::Title.like(req.message.text)
    //     )
    //     .one(&state.conn).await {
    //         Ok(val) => val,
    //         Err(error) => {
    //             return Response::builder()
    //                 .status(StatusCode::NOT_FOUND)
    //                 .body(());
    //         }
    //     };

    let type_model: types::Model = match types.into_iter().filter(|type_| type_.title == req.message.text).next() {
        Some(model) => model,
        None => {
            match tg_bot.mark_as_handled(
                req.update_id
            ) {
                Ok(_) => (),
                Err(_) => panic!("Redis write error, halting via panic!")
            }
            return Response::builder()
                .status(StatusCode::NOT_FOUND)
                .body(());
        }
    };

    let shitpost = match get_random_video_entry_of_type(
        &state.conn, type_model.id).await
        {
            Ok(val) => val,
            Err(_) => {
                match tg_bot.mark_as_handled(
                    req.update_id
                ) {
                    Ok(_) => (),
                    Err(_) => panic!("Redis write error, halting via panic!")
                }
                return Response::builder()
                    .status(StatusCode::NOT_FOUND)
                    .body(());
            }
        };

    match tg_bot.send_message(
        req.message.chat.id, &shitpost.url, &kb_markup
    ).await {
        Ok(_) => (),
        Err(_) => {
            match tg_bot.mark_as_handled(
                req.update_id
            ) {
                Ok(_) => (),
                Err(_) => panic!("Redis write error, halting via panic!")
            }
            return Response::builder()
                .status(StatusCode::NOT_FOUND)
                .body(());
        }
    };

    match tg_bot.mark_as_handled(req.update_id) {
        Ok(_) => {
            return Response::builder()
                    .status(StatusCode::OK)
                    .body(());
        },
        Err(_) => panic!("Redis write error, halting via panic!")
    }
}